/*
 * $Id: imatrix.c,v 1.2 1997/01/07 16:52:45 veran Exp $
 */

#include <malloc.h>
#include <stdio.h>

#include <math_utils.h>
#include <utilities.h>

/*
 * imatrix()
 *
 * memory allocation for a matrix of integers
 * returns m so that m[nrl][ncl],...,m[nrh][nch] are the valid locations
 *
 * last modified:  5/2/92
 */

int **
imatrix(int nrl, int nrh, int ncl, int nch)
{
  int i, n_cols, n_rows, total_pts;
  int **m;

  n_cols = nch-ncl+1;
  n_rows = nrh-nrl+1;
  if (n_rows<=0 || n_cols<=0) return(NULL);
  total_pts = n_cols*n_rows;
  
  if ( (m = (int **) malloc( (unsigned) (n_rows * sizeof(int *)))) == NULL)
    {
      system_mess_proc(1,"imatrix: memory allocation failure!");
    }
  else
    {
      if ( (m[0] = (int *) malloc( (unsigned) (total_pts * sizeof(int)))) == NULL)
	{
	  free(m);
	  m = NULL;
	  system_mess_proc(1,"imatrix: memory allocation failure!");
	}
      else
	{
	  m[0] = m[0] - ncl;
	  for (i=1; i<n_rows; i++) m[i] = m[i-1] + n_cols;
	  m = m-nrl;
	}
    }
  return(m);
}



/*
 * free_imatrix()
 *
 * frees memory allocated by dmatrix()
 *
 * last modified: 5/92
 */
void
free_imatrix(int **m, int nrl, int nrh, int ncl, int nch)
{
  if (m==NULL) return;
  free( (char *) (m[nrl] + ncl) );
  free( (char *) (m + nrl) );
}



